# -*- encoding: utf-8 -*-

import datetime
from django.contrib import admin
from django.contrib import messages
from django.shortcuts import redirect
from models import AUFMetadata

AUF_METADATA_READONLY_FIELDS = ('supprime',
                       'date_creation',
                       'user_creation',
                       'date_modification',
                       'user_modification',
                       'date_activation',
                       'user_activation',
                       'date_desactivation',
                       'user_desactivation', )

AUF_METADATA_FIELDSET_FIELDS = ('actif', 'date_creation', 'user_creation',
                       'date_modification', 'user_modification',
                       'date_activation', 'user_activation',
                       'date_desactivation', 'user_desactivation',)

class AUFMetadataInlineAdminMixin(object):
    exclude = AUF_METADATA_READONLY_FIELDS + ('actif', )


class AUFMetadataAdminMixin(object):
    """
    Surcharge l'admin de base, pour setter automatiquement les metadata.
    """
    readonly_fields = AUF_METADATA_READONLY_FIELDS
    fieldsets = (
        ('Metadata', {
            'classes': ('collapse',),
            'fields': AUF_METADATA_FIELDSET_FIELDS,
        }),
    )

    def desactiver(modeladmin, request, queryset):
      """
      Passe à actif = False tous les objets du QS
      """
      selected = request.POST.getlist(admin.ACTION_CHECKBOX_NAME)
      queryset.update(actif=False)
      messages.add_message(request, messages.INFO, 'Les %s ont été désactivés.' % queryset.model._meta.verbose_name_plural)
      info = queryset.model._meta.app_label, queryset.model._meta.module_name
      return redirect('admin:%s_%s_changelist' % info)
    desactiver.short_description = u"Désactiver les éléments sélectionnés"

    def save_model(self, request, obj, form, change):
        if obj.user_creation is None:
            obj.user_creation = request.user
            obj.date_creation = datetime.datetime.now()
        obj.user_modification = request.user
        obj.date_modification = datetime.datetime.now()
        initial = getattr(form, 'initial', None)
        if initial:
            if form.initial['actif'] is True and obj.actif is False:
                obj.user_desactivation = request.user
                obj.date_desactivation = datetime.datetime.now()
            if form.initial['actif'] is False and obj.actif is True:
                obj.user_activation = request.user
                obj.date_activation = datetime.datetime.now()
        obj.save()

    def has_metadata(self, instance):
        """
        Test si une instance de modele hérite bien de AUFMetadata.
        """
        test = getattr(instance, 'supprime', None)
        return test is not None

    def save_formset(self, request, form, formset, change):
        """
        Mettre les metadata si l'objet est créé ou modifié par inline.
        """
        instances = formset.save(commit=False)
        for instance in instances:
            if self.has_metadata(instance):
                self.save_model(request, instance, formset.form, change)           
            else:
                instance.save()
        formset.save_m2m()
